#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import re
import subprocess

TVE_REBOOT_DIR = '/config/_reboot/tve/now'

def request_tve_reboot():
    # Creation of TVE_REBOOT_DIR directory triggers the restart.
    # If the dir is present then remove it first, then create it. Otherwise the trigger will not sense the creation
    if os.path.exists(TVE_REBOOT_DIR):
        os.rmdir(TVE_REBOOT_DIR)

    # Create dirs
    os.makedirs(TVE_REBOOT_DIR)

def is_svc_masked(svc):
    try:
        state = subprocess.check_output(['systemctl', 'is-enabled', svc])
        if state.decode().rstrip() == 'masked':
            return True
    except subprocess.CalledProcessError as e:
        if e.output.decode().rstrip() == 'masked':
            return True

    return False

def get_hw_type():
    if not os.path.isfile('/config/.hwtype'):
        return 'intel'
    with open('/config/.hwtype', 'r') as hw_type_file:
        return hw_type_file.readline().splitlines()[0]

def shell_return_ok(self, cmd):
    try:
        self.execute_shell_command(cmd)
    except Exception as ex:
        self.logger.error(ex)
        return False
    return True

def get_ifname(self, name, vlan=None):
    ifname = name

    if name.find("data") > -1:
        ifname = "br-{}".format(name)

    if 'lte' in name:
        ifname = "br-{}".format(name)

    if 'wifi' in name:
        ifname = "br-{}".format(name)

    if vlan is not None and vlan != 0:
        if self.get_network_mode() == 'vpp':
            return ifname
        ifname = "{}.{}".format(ifname, vlan)
    return ifname

def set_pci_whitelist(self, action, devname, physical_network, current_list):
    import ast
    import json
    try:
        current_list = ast.literal_eval(str(current_list).strip())
    except Exception as ex:
        self.logger.error("Error parsing PCI current list '{}'".format(current_list))
        self.logger.error(ex)
        current_list = []
    if action == 'add':
        new_pci = {"devname": devname, "physical_network": physical_network}
        current_list.append(new_pci)
    elif action == 'remove':
        current_list[:] = [pci for pci in current_list if pci['devname'] != devname]
    return json.dumps(current_list)

def init_pci_data(self):
    self.pciids = dict()
    self.igb_vfs = dict()
    self.initramfs_changed = False
    self.igb_recalc_needed = False
    lines = ""

    if os.path.exists("/etc/udev/rules.d/70-persistent-net.rules"):
        with open("/etc/udev/rules.d/70-persistent-net.rules", "r") as f:
            lines = f.readlines()

    for line in lines:
        if 0 == len(line.strip()):
            continue
        line = line.replace('==', '=')
        line = line.replace(' ', '')
        rules = line.strip().split(',')
        rules_dict = dict()
        for rule in rules:
            (key, val) = tuple(rule.split('='))
            rules_dict[key] = val.replace('\"', '')

        if 'net' == rules_dict.get('SUBSYSTEM', 'none') and 'none' != rules_dict.get('NAME', 'none'):
            name = rules_dict.get('NAME', 'none')
            pci = rules_dict.get('KERNELS', 'none')
            tempdict = {name: pci}
            tempdict.update(self.pciids)
            self.pciids = tempdict

    for name, pci in self.pciids.items():
        if os.path.exists("/sys/bus/pci/drivers/igb/%s" % pci):
            self.igb_vfs[name] = 0
        else:
            if 0 == os.system('/sbin/dpdk-devbind --status-dev net | grep "unused=.*igb.*$" | grep "%s" -q' % pci):
                self.igb_vfs[name] = 0

def set_igb_param(self, name, mode):
    if name not in self.igb_vfs:
        return
    if mode == 'sr-iov':
        self.igb_vfs[name] = 7
    else:
        self.igb_vfs[name] = 0

    self.igb_recalc_needed = True

def recalc_igb_params(self):
    self.igb_recalc_needed = False
    max_vfs = None
    only_zeroes = True
    for n, vfs in self.igb_vfs.items():
        if max_vfs:
            max_vfs = "%s,%s" % (max_vfs, vfs)
        else:
            max_vfs = "%s" % (vfs)
        if vfs > 0:
            only_zeroes = False

    max_vfs_opt = "options igb max_vfs=%s" % max_vfs
    need_update = False
    try:
        optfile = open("/etc/modprobe.d/igb.conf", "r")
        cur_opt = optfile.read()
        optfile.close()
        if max_vfs_opt != cur_opt:
            if only_zeroes and bool(re.match('^options igb max_vfs=[0,]+$', cur_opt)):
                need_update = False
            else:
                need_update = True
    except:
        need_update = True
    try:
        if need_update:
            optfile = open("/etc/modprobe.d/igb.conf", 'w')
            optfile.write("options igb max_vfs=%s" % max_vfs)
            optfile.close()
            os.system('update-initramfs -u')
            self.initramfs_changed = True
    except:
        self.logger.error("Couldn't update \"/etc/modprobe.d/igb.conf\" file.")

def tag_interface(iface, vlan):
    if vlan is not None and vlan != 0:
        iface += '.' + str(vlan)
        return iface
    return iface
